/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-imagis.imag.fr/Membres/Yannick.Legoc/X3D/


/// X3DToolKit is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.

/// X3DToolKit is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.

/// You should have received a copy of the GNU General Public License
/// along with X3DToolKit; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
//                            NurbsSurface.h                                  //
//////////////////////////////////////////////////////////////////////////////// 

#ifndef NURBSSURFACE
#define NURBSSURFACE

#include "X3DTypes.h"
#include "X3DParametricGeometryNode.h"

namespace X3DTK {
namespace X3D {

class X3DTextureCoordinateNode;

/// Base class of the <a href="../../doc/X3DSpecs/Part01/components/NURBS.html">
/// NURBS component</a> defining a NurbsSurface
/// (<a href="../../doc/X3DSpecs/Part01/components/NURBS.html#NurbsSurface">
/// documentation</a>).

class NurbsSurface : public X3DParametricGeometryNode
{
public:
  /// Constructor.
  NurbsSurface();
  /// Constructs a NurbsSurface from it attribute.
  NurbsSurface(const MFVec3f                  &controlPoint,
               const X3DTextureCoordinateNode *texCoord,
               SFInt32                        uTesselation,
               SFInt32                        vTesselation,
               const MFDouble                 &weight,
               SFBool                         ccw,
               SFBool                         solid,
               SFInt32                        uDimension,
               SFInt32                        vDimension,
               const MFDouble                 &uKnot,
               const MFDouble                 &vKnot,
               SFInt32                        uOrder,
               SFInt32                        vOrder);
  /// Clones the node.
  virtual SFNode clone() const;
  /// Virtual destructor.
  virtual ~NurbsSurface();

  /// Sets the controlPoint attribute.
  void setControlPoint(const MFVec3f &controlPoint);
  /// Sets the texCoord attribute.
  void setTexCoord(const X3DTextureCoordinateNode *texCoord);
  /// Sets the uTesselation attribute.
  void setUTesselation(SFInt32 uTesselation);
  /// Sets the vTesselation attribute.
  void setVTesselation(SFInt32 vTesselation);
  /// Sets the weight attribute.
  void setWeight(const MFDouble &weight);
  /// Sets the ccw attribute.
  void setCcw(SFBool ccw);
  /// Sets the solid attribute.
  void setSolid(SFBool solid);
  /// Sets the uDimension attribute.
  void setUDimension(SFInt32 uDimension);
  /// Sets the vDimension attribute.
  void setVDimension(SFInt32 vDimension);
  /// Sets the uKnot attribute.
  void setUKnot(const MFDouble &uKnot);
  /// Sets the vKnot attribute.
  void setVKnot(const MFDouble &vKnot);
  /// Sets the uOrder attribute.
  void setUOrder(SFInt32 uOrder);
  /// Sets the vOrder attribute.
  void setVOrder(SFInt32 vOrder);

  /// Gets the controlPoint attribute.
  inline const MFVec3f &getControlPoint() const {return _controlPoint;};
  /// Gets the texCoord attribute.
  inline const SFNode &getTexCoord() const {return _texCoord;};
  /// Gets the uTesselation attribute.
  inline SFInt32 getUTesselation() const {return _uTesselation;};
  /// Gets the vTesselation attribute.
  inline SFInt32 getVTesselation() const {return _vTesselation;};
  /// Gets the weight attribute.
  inline const MFDouble &getWeight() const {return _weight;};
  /// Gets the ccw attribute.
  inline SFBool getCcw() const {return _ccw;};
  /// Gets the solid attribute.
  inline SFBool getSolid() const {return _solid;};
  /// Gets the uDimension attribute.
  inline SFInt32 getUDimension() const {return _uDimension;};
  /// Gets the vDimension attribute.
  inline SFInt32 getVDimension() const {return _vDimension;};
  /// Gets the uKnot attribute.
  inline const MFDouble &getUKnot() const {return _uKnot;};
  /// Gets the vKnot attribute.
  inline const MFDouble &getVKnot() const {return _vKnot;};
  /// Gets the uOrder attribute.
  inline SFInt32 getUOrder() const {return _uOrder;};
  /// Gets the vOrder attribute.
  inline SFInt32 getVOrder() const {return _vOrder;};

  /// Adds a child to the Children attribute. Returns TRUE if succeeds.
  virtual bool addChild(const SFAbstractNode &N);
  /// Sets the child. Behaves like addChild.
  virtual bool setChild(const SFAbstractNode &N);
  /// Removes the child if exists. Returns TRUE if succeeds.
  virtual bool removeChild(const SFAbstractNode &N);

  /// Loads the attributes from a X3DFileElement e.
  virtual void load(const X3DFileElement *element);
  /// Writes the attributes of the node.
  virtual SFString write() const;

protected:
  /// Copy constructor.  
  NurbsSurface(const NurbsSurface &N);

private:
  /// Set of control points of dimension _uDimension * _vDimension.
  MFVec3f _controlPoint;
  /// Additional information on how to generate texture coordinates.
  SFNode _texCoord;
  /// Tesselation in the u direction.
  SFInt32 _uTesselation;
  /// Tesselation in the v direction.
  SFInt32 _vTesselation;
  /// Weight values assigned to each control point.
  MFDouble _weight;
  /// Ccw attribute.
  SFBool _ccw;
  /// Solid attribute.
  SFBool _solid;
  /// Number of control points in the u direction.
  SFInt32 _uDimension;
  /// Number of control points in the v direction.
  SFInt32 _vDimension;
  /// Knot vector in the u direction.
  MFDouble _uKnot;
  /// Knot vector in the v direction.
  MFDouble _vKnot;
  /// Order in the u direction.
  SFInt32 _uOrder;
  /// Order in the v direction.
  SFInt32 _vOrder;
};

}
}

#endif
